import java.net.*;
import java.io.*;
import java.util.*;

class ChatServer {
	public static void main(String[] args) throws Exception {
		ChatServer server = ChatServer.getInstance();
		server.accept();
	}

	private static ChatServer instance;
	private ServerSocket ss;
	private int count;
	private Vector<ChatClientUser> cs;

	private ChatServer() throws IOException {
		ss = new ServerSocket(2345);
		cs = new Vector<ChatClientUser>();
	}

	public static ChatServer getInstance() {
		if (instance == null) {
			try { instance = new ChatServer(); }
			catch(Exception err) { System.out.println(err); }
		}
		return instance;
	}
	public void accept() throws IOException {
		while (true) {
			Socket socket = ss.accept();
			InetAddress addr = socket.getInetAddress();

			System.out.println("host = " + addr.getHostName());
			System.out.println("ip = " + addr.getHostAddress());

			cs.add(new ChatClientUser("user" + count, socket));
			count++;
		}
	}
	public void notify(ChatClientUser src, String msg) throws IOException {
		System.out.println(src.getName() + ":" + msg);
		if (msg == null || msg.equals("/q")) {
			for(int i = 0 ; i < cs.size() ; i++) {
				ChatClientUser user = cs.get(i);
				user.sendMessage("system", src.getName() + "ؒf܂");
			}
			cs.remove(src);
			src.getSocket().close();
		}
		else {
			for(int i = 0 ; i < cs.size() ; i++) {
				ChatClientUser user = cs.get(i);
				user.sendMessage(src.getName(), msg);
			}
		}
	}
}

class ChatClientUser implements Runnable {
	String name;
	Socket socket;

	public ChatClientUser(String name, Socket socket) {
		this.name = name;
		this.socket = socket;
		Thread thread = new Thread(this);
		thread.start();
	}
	public Socket getSocket() { return this.socket; }
	public String getName() { return this.name; }
	public void sendMessage(String name, String msg) throws IOException {
		OutputStream stream = socket.getOutputStream();
		PrintWriter writer = new PrintWriter(stream);
		writer.println(name + ":" + msg);
		writer.flush();
	}
	public void run() {
		try {
			InputStream stream = socket.getInputStream();
			BufferedReader reader = new BufferedReader(
				new InputStreamReader(stream)
			);
			while(!socket.isClosed()) {
				String msg = reader.readLine();
				ChatServer.getInstance().notify(this, msg);
			}
		}
		catch(Exception err) { System.out.println(err); }
	}
	public void close() throws IOException {
		socket.close();
	}
}