import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

public class Shooting extends JFrame implements KeyListener, ActionListener {
	private static final int WIDTH = 640, HEIGHT = 480;

	public static void main(String[] args) {
		JFrame frame = new Shooting();
		frame.setBounds(0 , 0 , WIDTH + 20 , HEIGHT + 40);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private Random random;
	private javax.swing.Timer timer;
	private ShootingPane shootingPane;
	private ShootingIcon userIcon;
	private Image userImage, enemyImage, userMissileImage, enemyMissileImage;

	public Shooting() {
		random = new Random(System.currentTimeMillis());

		shootingPane = new ShootingPane();
		shootingPane.setBorder(new LineBorder(Color.BLACK));
		shootingPane.setBounds(0, 0, WIDTH, HEIGHT);
		getContentPane().setLayout(null);
		getContentPane().add(shootingPane, BorderLayout.CENTER);

		userImage = getToolkit().createImage("user.gif");
		enemyImage = getToolkit().createImage("enemy.gif");
		userMissileImage = getToolkit().createImage("um.gif");
		enemyMissileImage = getToolkit().createImage("em.gif");

		userIcon = new ShootingIcon(
			new ImageIcon(userImage),
			WIDTH / 2 - userImage.getWidth(this) / 2,
			HEIGHT - userImage.getHeight(this)
		);

		shootingPane.setUserIcon(userIcon);
		setTitle("V[eBOQ[");

		timer = new javax.swing.Timer(50, this);
		timer.start();

		addKeyListener(this);
	}

	//{^ꂽ
	private void onLeftButton() {
		int x = userIcon.getX() - 10;
		if (x <= 0) return;

		userIcon.setX(x);	//ֈړ
		repaint();
	}
	//E{^ꂽ
	private void onRightButton() {
		int x = userIcon.getX() + 10;
		if (x + userIcon.getWidth() >= WIDTH) return;

		userIcon.setX(x);	//Eֈړ
		repaint();
	}
	//Xy[X{^ꂽ
	private void onSpaceButton() {
		java.util.List<ShootingIcon> list = shootingPane.getUserMissles();
		if (list.size() >= 3) return;	//3ȏ͌ĂȂ

		Icon icon = new ImageIcon(userMissileImage);

		ShootingIcon mi = new ShootingIcon(icon,
			userIcon.getX() + (userIcon.getWidth() / 2) - icon.getIconWidth() / 2,
			userIcon.getY()
		);
		list.add(mi);	//V~TCˏo
		repaint();
	}

	public void keyTyped(KeyEvent e) {}
	public void keyPressed(KeyEvent e) {
		switch(e.getKeyCode()) {
		case KeyEvent.VK_LEFT:
			onLeftButton();
			break;
		case KeyEvent.VK_RIGHT:
			onRightButton();
			break;
		case KeyEvent.VK_SPACE:
			onSpaceButton();
			break;
		}
	}
	public void keyReleased(KeyEvent e) {}

	public void actionPerformed(ActionEvent e) {
		java.util.List<ShootingIcon> el = shootingPane.getEnemys();
		java.util.List<ShootingIcon> uml = shootingPane.getUserMissles();
		java.util.List<ShootingIcon> eml = shootingPane.getEnemyMissles();

		//VG𐶐
		if (el.size() < 5) {
			ImageIcon imageIcon = new ImageIcon(enemyImage);
			ShootingIcon icon = new ShootingIcon(
				imageIcon, random.nextInt(WIDTH - imageIcon.getIconWidth()), 0
			);
			el.add(icon);
		}
		//mɂēG~TCˏo
		if (eml.size() < 5 && random.nextInt(100) < 10) {
			int index = random.nextInt(el.size());
			ShootingIcon enemy = el.get(index);
			ShootingIcon missile = new ShootingIcon(
				new ImageIcon(enemyMissileImage),
				enemy.getX() + enemy.getWidth() / 2 - enemy.getWidth() / 2,
				enemy.getY() + enemy.getHeight()
			);
			eml.add(missile);
		}

		//GIuWFNgړ
		for(int i = 0 ; i < el.size() ; i++) {
			ShootingIcon icon = el.get(i);
			int y = icon.getY() + 4;
			int x = icon.getX();

			if (i % 2 == 0) x += 2;
			else x -= 2;

			if (x < 0) {
				el.remove(i);
				continue;
			}
			else if(x + icon.getWidth() > WIDTH) {
				el.remove(i);
				continue;
			}
			else if(y > HEIGHT) {
				el.remove(i);
				continue;
			}

			icon.setX(x);
			icon.setY(y);
		}

		//̃~TCړ
		for(int i = 0 ; i < uml.size() ; i++) {
			ShootingIcon icon = uml.get(i);
			int y = icon.getY() - 10;
			if (y <= 0) uml.remove(i);
			else icon.setY(y);
		}

		//G~TCړ
		for(int i = 0 ; i < eml.size() ; i++) {
			ShootingIcon icon = eml.get(i);
			int y = icon.getY() + 5;
			if (y >= HEIGHT) eml.remove(i);
			else icon.setY(y);
		}

		//G̓蔻
		for(int i = 0 ; i < el.size() ; i++) {
			ShootingIcon enemy = el.get(i);
			for(int j = 0 ; j < uml.size() ; j++) {
				ShootingIcon missile = uml.get(j);
				if (enemy.getBounds().intersects(missile.getBounds())) {
					//Gɒe
					uml.remove(j);
					el.remove(i);
					break;
				}
			}
		}

		//̓蔻
		for(int i = 0 ; i < el.size() ; i++) {
			ShootingIcon enemy = el.get(i);
			if (enemy.getBounds().intersects(userIcon.getBounds())) {
				//GɐڐG
				timer.stop();
			}
		}
		for(int i = 0 ; i < eml.size() ; i++) {
			ShootingIcon missile = eml.get(i);
			if (missile.getBounds().intersects(userIcon.getBounds())) {
				//~TCɐڐG
				timer.stop();
			}
		}

		repaint();
	}
}

class ShootingIcon {
	private Icon icon;
	private int x, y;

	ShootingIcon() { this(null, 0, 0); }
	ShootingIcon(Icon icon) { this(icon, 0, 0); }
	ShootingIcon(Icon icon, int x, int y) {
		this.icon = icon;
		this.x = x;
		this.y = y;
	}

	//ACR
	public void setIcon(Icon icon) { this.icon = icon; }
	public Icon getIcon() { return icon; }

	//W
	public void setX(int x) { this.x = x; }
	public int getX() { return x; }
	public void setY(int y) { this.y = y; }
	public int getY() { return y; }

	//TCY
	public int getWidth() { return icon.getIconWidth(); }
	public int getHeight() { return icon.getIconHeight(); }

	//Rectangle IuWFNgԂ
	public Rectangle getBounds() {
		return new Rectangle(getX(), getY(), getWidth(), getHeight());
	}
}

class ShootingPane extends JComponent {
	private ShootingIcon user;
	private Vector<ShootingIcon> userMissiles;

	private Vector<ShootingIcon> enemys;
	private Vector<ShootingIcon> enemyMissiles;

	public ShootingPane() {
		userMissiles = new Vector<ShootingIcon>();
		enemys = new Vector<ShootingIcon>();
		enemyMissiles = new Vector<ShootingIcon>();
		
	}

	public void setUserIcon(ShootingIcon user) { this.user = user; }
	public ShootingIcon getUserIcon() { return user; }

	public java.util.List<ShootingIcon> getUserMissles() {
		return userMissiles;
	}
	public java.util.List<ShootingIcon> getEnemyMissles() {
		return enemyMissiles;
	}
	public java.util.List<ShootingIcon> getEnemys() {
		return enemys;
	}

	public void paintComponent(Graphics g) {
		if (user != null) //@`
			user.getIcon().paintIcon(this, g, user.getX(), user.getY());

		//G`
		for(int i = 0 ; i < enemys.size() ; i++) {
			ShootingIcon enemy = enemys.get(i);
			enemy.getIcon().paintIcon(this, g, enemy.getX(), enemy.getY());
		}

		//̃~TC`
		for(int i = 0 ; i < userMissiles.size() ; i++) {
			ShootingIcon missile = userMissiles.get(i);
			missile.getIcon().paintIcon(this, g, missile.getX(), missile.getY());
		}

		//G̃~TC`
		for(int i = 0 ; i < enemyMissiles.size() ; i++) {
			ShootingIcon missile = enemyMissiles.get(i);
			missile.getIcon().paintIcon(this, g, missile.getX(), missile.getY());
		}
	}
}